"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeProject = void 0;
const fs_1 = __importDefault(require("fs"));
const addToGitIgnore_1 = require("../helpers/addToGitIgnore");
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const loadExistingFile_1 = require("../helpers/loadExistingFile");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
const isomorphic_git_1 = require("isomorphic-git");
const path_1 = require("path");
const initializeProject = (targetDir, name, version, description, authorName, authorEmail, license) => __awaiter(void 0, void 0, void 0, function* () {
    const previousPackage = JSON.parse((yield (0, loadExistingFile_1.loadExistingFile)(targetDir, "package.json")) || "{}");
    const packageJson = Object.assign(Object.assign(Object.assign(Object.assign({}, previousPackage), { name,
        version,
        description }), (authorName || authorEmail ? { author: { name: authorName, email: authorEmail } } : {})), { repository: yield getRepositoryInformation(targetDir), license });
    yield (0, writeAndAddFile_1.writeAndAddFile)(targetDir, "package.json", JSON.stringify(packageJson, null, 2));
    yield (0, addToGitIgnore_1.addToGitIgnore)(targetDir, "node", ["node_modules", "yarn-error.log"]);
    yield (0, commitWithAuthor_1.commitWithAuthor)(targetDir, "Initialize node project");
});
exports.initializeProject = initializeProject;
const getRepositoryInformation = (targetDir) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const remotes = yield (0, isomorphic_git_1.listRemotes)({ fs: fs_1.default, dir: targetDir });
    const origin = (_a = remotes.find(remote => remote.remote === "origin")) === null || _a === void 0 ? void 0 : _a.url;
    if (!origin) {
        throw new Error("Currently you need a git remote");
    }
    const gitRoot = yield (0, isomorphic_git_1.findRoot)({ fs: fs_1.default, filepath: targetDir });
    const absoluteTargetDir = (0, path_1.normalize)((0, path_1.resolve)(process.cwd(), targetDir));
    const absoluteGitRootDir = (0, path_1.normalize)((0, path_1.resolve)(process.cwd(), gitRoot));
    const projectLocationRelativeToGitRoot = (0, path_1.relative)(absoluteGitRootDir, absoluteTargetDir);
    const httpsGitOrigin = origin.replace(/git@([^:]+):(.+)\.git/, "https://$1/$2");
    const gitDirectory = (0, path_1.normalize)(projectLocationRelativeToGitRoot) === "." ? undefined : (0, path_1.normalize)(projectLocationRelativeToGitRoot);
    return {
        type: "git",
        url: httpsGitOrigin,
        projectLocationRelativeToGitRoot: gitDirectory,
    };
});
